/*
 * mlex.h - lexical analyser header
 *
 * Mikhail Ilyin, <springheads@gmail.com>
 * Grisha Sitkarev, <sitkarev@unixkomi.ru>
 *
 * Copyright (c) 2010
 *
 * This is a free software and you can redistribute it and
 * use under the terms of GNU Public License version 2 or
 * any later version.
 */
 
#ifndef MLEX_H_
#define MLEX_H_

typedef enum {
	TOKEN_UNKNOWN = 0,
	TOKEN_INTEGER,
	TOKEN_REAL,
	TOKEN_CHAR,
	TOKEN_STRING,
	TOKEN_ID,
	TOKEN_ASTERISK,
	TOKEN_SLASH,
	TOKEN_PLUS,
	TOKEN_MINUS,
	TOKEN_PERCENT,
	TOKEN_DOLLAR,
	TOKEN_SEMICOLON,
	TOKEN_COMMA,
	TOKEN_LBRACE,
	TOKEN_RBRACE, /* 15 */
	TOKEN_LBRACKET,
	TOKEN_RBRACKET,
	TOKEN_LPARENTH,
	TOKEN_RPARENTH,
	TOKEN_EQUALITY, /* 20 */
	TOKEN_LT,
	TOKEN_GT,
	TOKEN_LE,
	TOKEN_GE,
	TOKEN_NE,
	TOKEN_EQ,
	TOKEN_OR,
	TOKEN_AND,
	TOKEN_NOT,
	TOKEN_DOT, /* 30 */
	/* keyword tokens */
	TOKEN_DECL_INT, 
	TOKEN_DECL_REAL,
	TOKEN_DECL_CHAR,
	TOKEN_DECL_STRING,
	TOKEN_DECL_VOID,
	TOKEN_IF,
	TOKEN_ELSE,
	TOKEN_WHILE,
	TOKEN_FOR,
	TOKEN_BREAK,
	TOKEN_CONTINUE,
	TOKEN_RETURN,
	TOKEN_MODULE,
	TOKEN_EXTERN,
	TOKEN_FMT,
	TOKEN_BYREF,
	TOKEN_AMPERSAND,
	/* Reserve place for additional tokens. */
	TOKEN_EOF = 255
} token_t;

struct mlex {
	token_t			token;
	union {
		int		integer;
		float		real;
		char		ch;
		char		*id;
		char		*str;
	};
};

extern struct mlex mlex;

token_t mlex_get_next_token();

unsigned int mlex_line_number();

const char *mlex_file_name();

void mlex_file_setup(char *begin, char *end);

void mlex_file_finalize();

#endif
